---------------------------------------------------------
--(copy left) Arthur Øvergaard
--Trondheim June 16, 2009
--
--Implements Newton-Raphson simulation model
---------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.ALL;

library work;
use work.NewtonRaphsonSimComponents.all;
use work.txt_util.all;


entity sim_newton is 
	port(	dividend : in std_logic_vector(63 downto 0);
			divisor : in std_logic_vector(63 downto 0);
			clock : in std_logic;
			quotient : out std_logic_vector(63 downto 0);
			rest : out std_logic_vector(63 downto 0));			
end sim_newton;

architecture simulator of sim_newton is

begin

	process(clock)
		variable shift : integer;
		variable divisor_norm : std_logic_vector(63 downto 0);--<<1:63>>

		variable res0 : std_logic_vector(3 downto 0);--First 
		variable res1 : std_logic_vector(7 downto 0);
		variable res2 : std_logic_vector(15 downto 0);
		variable res3 : std_logic_vector(31 downto 0);
		variable res4 : std_logic_vector(63 downto 0);

		variable quot : std_logic_vector(res4'length + dividend'length - 1 downto 0);
		variable quot_trunc : std_logic_vector(quotient'range);

		variable app_divd : std_logic_vector(quot_trunc'length + divisor'length - 1 downto 0);
		variable rest_v : std_logic_vector(63 downto 0);
		
		variable one : std_logic_vector(63 downto 0);
		
		variable adj_count : integer := 0;
		variable s : string(1 to 10);
		
	begin
		if clock'event and clock = '1' then
			
			one := (others => '0');
			one(one'high) := '1';
	
			--steg 1: control input
			if ieee.std_logic_arith.unsigned(divisor) = 0 then
				report "Division by zero";
			elsif dividend = divisor then
				quotient <= (0 => '1', others => '0');
				rest <= (others => '0');
			else
		
				--steg 2: Normalize divisor
				divisor_norm := divisor;
				norm(divisor_norm, shift);
			
				if divisor_norm = one then
					quot := (others => '0');
					quot(dividend'range) := dividend;
					quot_trunc := quot(quot'high - shift  - 1 downto quot'length - quot_trunc'length - shift - 1);
				else
					--steg 3: tabell oppslag
					res0 := table_loockup(divisor_norm(divisor_norm'high downto divisor_norm'length - res0'length - 1), res0'length);

					--steg 4: first NewtonRaphson iterasjon
					res1 := NewtonRaphsonStage(divisor_norm, res0);
					--steg 5: second NewtonRaphson iterasjon
					res2 := NewtonRaphsonStage(divisor_norm, res1);
					res3 := NewtonRaphsonStage(divisor_norm, res2);
					res4 := NewtonRaphsonStage(divisor_norm, res3);
					--steg 6: regn quotient			
					quot :=  dividend * res4;
					for i in 0 to 63 - (shift+1) loop
						quot := '0' & quot(quot'high downto 1);
					end loop;
					--quot := quot + (quot'high downto (quot'high - quot_trunc'length - 1) => '0', (quot'high - quot_trunc'length - 2) => '1', (quot'high - quot_trunc'length - 3) downto 0 => '0');
					quot_trunc := quot(quot'high downto quot'length - quot_trunc'length);	   
				 
				end if; --if divisor_norm = one then
			
				app_divd := (quot_trunc * divisor);
				rest_v := dividend - app_divd(rest_v'range);
			
				if rest_v >= divisor then
					quotient <= quot_trunc + '1';
					rest <= rest_v - divisor;
					adj_count := adj_count + 1;
					s := str(adj_count, 10);
					report "Adjust:" & s;
				else
					quotient <= quot_trunc;
					rest <= rest_v;
				end if;				

			end if;
	   end if;		
	end process;
	
end simulator;
