#pragma once

#include <fstream>

class Decomposition
{
	public:
		Decomposition(int Wi_, int Wo_, int TOs, int alpha_, const int* gamma_, const int * beta_);
		Decomposition(const Decomposition& ref);
		Decomposition(std::fstream& file);
		~Decomposition();
		
		
		void SaveToFile(std::fstream& fileRef)const;
		
		void Print(bool symmetry = true)const;		//Print the decomposition 
		int Entrys(bool)const;			//Calculate number of entrys for this decomposition
		
		void Verify(bool verbose)const;

		
		static bool CopmearEntrysSym(Decomposition& a, Decomposition& b){return a.Entrys(true) < b.Entrys(true);}
		static bool CopmearEntrysNoSym(Decomposition& a, Decomposition& b){return a.Entrys(false) < b.Entrys(false);}
		static bool CopmearBitSizeSym(Decomposition& a, Decomposition& b){return a.CalcSize(true) < b.CalcSize(true);}
		static bool CopmearBitSizeNoSym(Decomposition& a, Decomposition& b){return a.CalcSize(false) < b.CalcSize(false);}
		
		//Accessor functons
		void getTIV(int* TIVbuffer, int & max_size)const;
		void getTO(int i, int* TIVbuffer, int & max_size, bool symmetry)const;
		int getAlpha()const{return alpha;}
		void getTOparam(int i, int& Gamma_, int &Beta_, int& P_)const;
		int getWidthTIV()const{return Wo + getGuardBits();} 
		int getWidthTO(int i)const;
		int getmTOs()const{return mTOs;}
		int getWo()const{return Wo;}
		int getWi()const{return Wi;}
		int getGuardBits()const;			//Calculate number of k guard bits needed to achive calculated 
		long double getMathApproxError()const{return ApproxError;}
		bool getErrorOK()const{return errorOK;}
		
		
		
	private:		//Private class functions
		
		inline long double f(long double x)const{	//The function we are calculating
			return 1/x;}
		
		long double calcMathApproxError()const;
		
		long double getRangeTO(int i)const;
		
		long double Delta(int i)const;
		long double Xright(unsigned int Ai, int i)const;
		long double Xleft(unsigned int Ai, int i)const;
		
		long double Slope(int Ai, int i)const;
		int CalcSize(bool symmetry)const;
		
		
	private:		//Private class atributs
	
		int Wi, Wo; 	///Input and output word width
		
		int mTOs; 		///Number of TO tebles
		
		int alpha; 				///Width of Part A
		int * gamma; 			///Width of TOi gamma index
		int * beta; 			///Width of TOi betha index
		int * p;				///Width of pi
		long double ApproxError;///The the error introdused due to mathematical approximation
		bool errorOK;
};
