#include <iostream>
#include <math.h>
#include "DecompositionSets.h"
#include "VHDLcodegen.h"
#include "Exception.h"


	
/**
 *
 *Generates a set of decompositions based on input parameters
 @param Wi width if the input word
 @param Wo width of the output word
 @param m number of TOs
 */
DecompositionSets::DecompositionSets(int Wi, int Wo, int m)
{
	TempData first(m);

	for(int A = 1 ; A <= Wi-m;A++)
	{
		first.alphaSize = A;
		SplitRest(Wi, Wo, Wi-A,m, first);
	}
}

int getNumber(std::string str, unsigned int&index){
	char num[5]={0};
	unsigned int i=0;
	
	if(index >= str.length())return 0;
	
	while(index < str.length() && str[index] < '0' && str[index] >'9')index++;
	
	while(i<4 && index<str.length()){
		if(str[index]>= '0' && str[index]<='9'){
			num[i++] = str[index];
			index++;
		}else{
			index++;
			break;
		}
	}
	return atoi(num);	
}

/**
 *Loades a set of decompositions from file
 */
DecompositionSets::DecompositionSets (std::fstream& file, fileFormat format)
{
	std::string line;
	bool has_start = false;
	int * TOSgamma = 0;
	int * TOSbeta = 0;
	int Wi, Wo, mTOs, alpha;
	unsigned int lineCount, number, index;	
	
	if(!file.is_open())throw Exception("File not open");
	
	if(format == BINARY){
		while(!file.eof()){
			Decompositions.push_back(Decomposition(file));
		}
		Decompositions.erase(--(Decompositions.end())); //???This is becous it loades a exstra dummy item
	}else{
		while(!file.eof()){
			getline(file, line);
			//std::cout << line << std::endl;
			if(line[0] == '#')continue;
			
			if(has_start){
				index = 0;
				if(lineCount == 0){
					number = 0;
					
					Wi = getNumber(line, index);
					Wo = getNumber(line, index);	
					mTOs = getNumber(line, index);	
					alpha = getNumber(line, index);
					
					TOSgamma = new int [mTOs];
					TOSbeta = new int [mTOs];
					lineCount++;
				}else{
					TOSgamma[mTOs - lineCount] = getNumber(line, index);
					TOSbeta[mTOs - lineCount] = getNumber(line, index);
					lineCount++;
					
					if(lineCount > (unsigned int)mTOs){
						//std::cout << "Test# Wi:" << Wi << " Wo:" << Wo << " m:" << mTOs << " alpha:" << alpha << std::endl;
						has_start = false;
						Decompositions.push_back(Decomposition(Wi, Wo, mTOs, alpha, TOSgamma, TOSbeta));
						delete [] TOSgamma;
						delete [] TOSbeta;
					}
				}				
			}else{
				if(line.find(".SET")!=std::string::npos)has_start = true;
				lineCount = 0;
			}
		}
	}
		

}


/**
 *Recursive the remaining part of B in to smaler parts
 @param Wi input data width
 @param Wo output data width
 @param RestSize size of remaining part
 @param parts number of parts to devide rest in
 @param ref referance to a decomposition where the start og B is devided
 */
void DecompositionSets::SplitRest(int Wi, int Wo, int RestSize, int parts, TempData ref)
{
	using namespace std;
		
	Decomposition * tempDecomp = 0;

	for(int gamma = 1; gamma <= ref.alphaSize ; gamma++)
	{
		ref.gamma[parts-1] = gamma;
		
		if(parts==1)
		{
			ref.beta[0] = RestSize;
		
			tempDecomp = new Decomposition(Wi, Wo, ref.num_TOs, ref.alphaSize, ref.gamma, ref.beta);
			if(tempDecomp->getErrorOK()){
				//tempDecomp->Print();
				Decompositions.push_back(*tempDecomp);
			}
			delete tempDecomp;
		}
		else
		{
			
			for(int Bi = RestSize - (parts-1); Bi > 0; Bi--)
			{
				ref.beta[parts-1] = Bi;
				SplitRest(Wi, Wo, RestSize - Bi, parts-1, ref); 
			}
		}
	}

}

/**
 *Sorts the compositions desending after selected methode
 @param method eigther by Entrys of total bit size
 */
void DecompositionSets::SortDecompositions(sortMethode method)
{
	switch(method){
		case ENTRYS : Decompositions.sort(Decomposition::CopmearEntrysSym); break;
		case BIT_SIZE : Decompositions.sort(Decomposition::CopmearBitSizeSym); break;
		default : return;
	}
}


/**
 *Keeps the count first decompositions rest is deleted
 @param count number of decompositions to keep
 */
void DecompositionSets::Crop(unsigned int count)
{
	std::list<Decomposition>::iterator it = Decompositions.begin();

	unsigned int i = 0;
	
	while(i<count && it != Decompositions.end()){
		++it; i++;
	}
	
	Decompositions.erase(it, Decompositions.end());
}




/**
 *write the decomposition set to a binary file for later loading
 */
void DecompositionSets::writeToFile(std::fstream & file)
{
	std::list<Decomposition>::iterator it = Decompositions.begin();
	
	while(it != Decompositions.end()){
		(*(it)).SaveToFile(file);
		it++;
	}
		
}

/**
 *Print all decompositions in the set
 */
void DecompositionSets::Print()const{
	std::list<Decomposition>::const_iterator it = Decompositions.begin();
	
	while(it != Decompositions.end()){
		try{(*(it)).Print();}
		catch(Exception ex){ex.Print();}
		it++;
	}		
}

void DecompositionSets::PrintRev()const{
	std::list<Decomposition>::const_reverse_iterator it = Decompositions.rbegin();
	
	while(it != Decompositions.rend()){
		try{(*(it)).Print();}
		catch(Exception ex){ex.Print();}
		it++;
	}		
}

void DecompositionSets::Verify(bool verbose)const{
	std::list<Decomposition>::const_iterator it = Decompositions.begin();
	
	while(it != Decompositions.end()){
		(*(it)).Verify(verbose);
		it++;
	}		
}

void DecompositionSets::GenerateVHDL(std::string path, int set)const
{
	int i = 0;
	std::list<Decomposition>::const_iterator it = Decompositions.begin();
	VHDLgen * gen;
	
	while(it != Decompositions.end() && i < set){
		it++;
	}
	
	if(it != Decompositions.end()){
		//(*it).Print();
		gen = new VHDLgen(*it);
		try{gen->Generate(path);}
		catch(Exception ex){ex.Print();}
		delete gen;
	}
}
