#pragma once
#include <fstream>
#include "Decomposition.h"
#include <list>

/**
 *Class that generates a set of decomositions based on  input and output width and number of tables
 */
class DecompositionSets
{
	public:
		enum sortMethode{
			ENTRYS = 0,
			BIT_SIZE = 1};
		enum fileFormat{
			BINARY = 0,
			TEXT = 1};
		
		DecompositionSets(int Wi, int Wo, int m);
		DecompositionSets(std::fstream & file, fileFormat format );
		
		void writeToFile(std::fstream & file);
		
		unsigned int Sets()const {return Decompositions.size();}
		
		//void RemoveInvalid(unsigned int Wo);
		void SortDecompositions(sortMethode method);
		void Crop(unsigned int count);
		
		void Print()const;
		void PrintRev()const;
		void Verify(bool verbose)const;
		
		void GenerateVHDL(std::string path, int set = 0)const;
	
	private:
		struct TempData{
			///Default constructor
			TempData(int number_of_tos){
				num_TOs = number_of_tos;
				gamma = new int[num_TOs];
				beta = new int[num_TOs];
				
				for(int i=0;i<num_TOs;i++){
					gamma[i] = 0;
					beta[i] = 0;
				}
			}
			
			///Copy constructor
			TempData(const TempData& other){
				num_TOs = other.num_TOs;
				alphaSize = other.alphaSize;
				gamma = new int[num_TOs];
				beta = new int[num_TOs];
				
				for(int i=0;i<num_TOs;i++){
					gamma[i] = other.gamma[i];
					beta[i] = other.beta[i];
				}
			}
				
			~TempData(){
				delete [] gamma;
				delete [] beta;
			}
			
			
			///number of tos in the decomposition
			int num_TOs;
			///size of the TIV address bus
			int alphaSize;
			///sise of the gamma address part of TO[i]
			int * gamma;
			///sise of the beta address part of TO[i]
			int * beta;
		};
		
		int Compear(Decomposition& a, Decomposition& b);
		void SplitRest(int Wi, int Wo, int RestSize, int parts, TempData ref);
		std::list<Decomposition> Decompositions;
		
};
