#include <iostream>
#include <string>
#include <fstream>
#include <list>
#include <math.h>

#include "Decomposition.h"
#include "DecompositionSets.h"
#include "Exception.h"


/**
 *Class for interpret input param
 @param argc, argv
 */ 
class InputDecoder
{
	public:
	
		InputDecoder(int argc, char ** argv);
	
		enum {
			NONE = 0x00,
			INPUT_FILE_B = 0x01,
			INPUT_FILE	= 0x02,
			INPUT_WIDTH = 0x04,
			OUTPUT1_FILE = 0x08,
			OUTPUT2_FILE = 0x10,
			VERBOSE		= 0x20,
			VERIFY		= 0x40
		};
		
		unsigned int flags;
		
		std::string inFilePath;
		std::string out1FilePath;
		std::string out2FilePath;
		bool verbose;
		unsigned int Wi, Wo, m;
				
	
};

InputDecoder::InputDecoder(int argc, char ** argv)
{
	using namespace std;
	
	bool hasInput = false;
	bool hasOutput1 = false;
	bool hasOutput2 = false;
	
	this->flags = NONE;
	
	int i = 1;
	if(argc == 1){//No arguments given
		std::cout << "usage:" << std::endl;
		std::cout << "{[-i inputfile] [-I input_width output_width TOs]}" << std::endl;
		std::cout << "[-o outputfile] [-O output_table_file]" << std::endl;
		std::cout << "[-v] verbose" << std::endl;
	}
	
	while(i < argc)
	{
		if(strcmp(argv[i], "-i") == 0){
			i++;
			if(hasInput){
				std::cout << "Multipel input param ignored" << std::endl;
			}else if(i<argc){
				inFilePath = argv[i++];
				hasInput = true;
				if(inFilePath.find(".txt") != std::string::npos)
					this->flags |= INPUT_FILE;
				else
					this->flags |= INPUT_FILE_B;
			}else{
				std::cout << "Missing input file" << std::endl;
				i++;
			}
		}else if(strcmp(argv[i], "-I") == 0){
			i++;
			if(hasInput){
				std::cout << "Multipel input param ignored" << std::endl;
			}else if(i + 3 <= argc){
				Wi = atoi(argv[i]);i++;
				Wo = atoi(argv[i]);i++;
				m = atoi(argv[i]);i++;
				hasInput = true;
				this->flags |= INPUT_WIDTH;
			}else{
				std::cout << "Missing input data" << std::endl;
				i++;
			}
		}else if(strcmp(argv[i], "-o") == 0){
			i++;
			if(hasOutput1){
				std::cout << "Multipel output param ignored" << std::endl;
			}else if(i <= argc){
				out1FilePath = argv[i];i++;
				hasOutput1 = true;
				this->flags |= OUTPUT1_FILE;
			}else{
				std::cout << "Missing output data" << std::endl;
				i++;
			}
		}else if(strcmp(argv[i], "-vhdl") == 0){
			i++;
			if(hasOutput2){
				std::cout << "Multipel output param ignored" << std::endl;
			}else if(i <= argc){
				out2FilePath = argv[i];i++;
				hasOutput2 = true;
				this->flags |= OUTPUT2_FILE;
			}else{
				std::cout << "Missing output data" << std::endl;
				i++;
			}
		}
		else if(strcmp(argv[i], "-v") == 0){
			this->flags |= VERBOSE;
			i++;
		}
		else if(strcmp(argv[i], "-V") == 0){
			this->flags |= VERIFY;
			i++;
		}else{
			std::cout << "'" << argv[i] << "' Ignored" << std::endl;
			i++;
		}			
	}
}

int main(int argc, char ** argv)
{
	
	DecompositionSets * theSets = 0;
	
	std::fstream * file = 0;
	
	InputDecoder input(argc, argv);

	
	if(input.flags & InputDecoder::INPUT_WIDTH){
		try{
			theSets = new DecompositionSets(input.Wi, input.Wo,  input.m);
			theSets->SortDecompositions(DecompositionSets::ENTRYS);
			//theSets->PrintRev();
			theSets->Crop(200);
			theSets->SortDecompositions(DecompositionSets::BIT_SIZE);
			theSets->Crop(100);
			//theSets->PrintRev();
			//theSets->GenerateVHDL();
		}catch(Exception ex){
			ex.Print();
		}catch(...){
		}
	
	}else if(input.flags & InputDecoder::INPUT_FILE){
		try{
			file = new std::fstream(input.inFilePath.c_str(), std::fstream::in);
			theSets = new DecompositionSets(*file, DecompositionSets::TEXT);
			file->close();
			delete file;
			file = 0;
		}catch(Exception ex){
			ex.Print();
		}catch(...){
			std::cout << "Undefined eror in txt file read" << std::endl;
		}
		
	}else if(input.flags & InputDecoder::INPUT_FILE_B){
		try{
			file = new std::fstream(input.inFilePath.c_str(), std::ifstream::binary | std::fstream::in);
			theSets = new DecompositionSets(*file, DecompositionSets::BINARY);
			file->close();
			delete file;
			file = 0;
		}catch(Exception ex){
			ex.Print();
		}catch(...){
			std::cout << "Undefined eror in txt file read" << std::endl;
		}
	}
	
	if(input.flags & InputDecoder::VERBOSE){
		try{
			theSets->PrintRev();
		}catch(Exception ex){
			ex.Print();
		}catch(...){
			std::cout << "Undefined eror in txt file read" << std::endl;
		}	
	}
	
	if(input.flags & InputDecoder::VERIFY){
		try{
			if(input.flags & InputDecoder::VERBOSE)
				theSets->Verify(true);
			else
				theSets->Verify(false);
		}catch(Exception ex){
			ex.Print();
		}catch(...){
			std::cout << "Undefined eror in txt file read" << std::endl;
		}	
	}
	
	if(input.flags & InputDecoder::OUTPUT1_FILE){
		file = new std::fstream(input.out1FilePath.c_str(), std::fstream::binary | std::fstream::out);
		theSets->writeToFile(*file);
		file->close();
		delete file;
	}
	
	if(input.flags & InputDecoder::OUTPUT2_FILE){
		theSets->GenerateVHDL(input.out2FilePath);
	}
	
	
	delete theSets;
	
	return 0;
}
