#include "VHDLcodegen.h"
#include <fstream>
#include <iostream>

int VHDLgen::class_count = 0;
std::map<int, std::string> VHDLgen::strings;

typedef std::pair<int, std::string> mypair ;

VHDLgen::VHDLgen(const Decomposition& source_):source(source_){
	if(class_count == 0){
		generateStringMap();
	}
	class_count++;
	
	userStrings.insert(mypair(ENTITY_NAME,"table_loockup"));
	userStrings.insert(mypair(ARCHITECTURE_NAME,"behaviour"));
	userStrings.insert(mypair(INPUT_NAME,"data_in"));
	userStrings.insert(mypair(OUTPUT_NAME,"data_out"));
	userStrings.insert(mypair(GENERIC_NAME,"width"));
}

VHDLgen::~VHDLgen(){
	class_count--;
	
	if(class_count == 0){
		generateStringMap();
	}
	strings.clear();
}

void VHDLgen::generateStringMap()
{
	strings.insert(mypair(ABS,"ABS"));
	strings.insert(mypair(ACCESS,"ACCESS"));
	strings.insert(mypair(AFTER,"AFTER"));
	strings.insert(mypair(ALIAS,"ALIAS"));
	strings.insert(mypair(ALL,"ALL"));
	strings.insert(mypair(AND,"AND"));
	strings.insert(mypair(ARCHITECTURE,"ARCHITECTIURE"));
	strings.insert(mypair(ARRAY,"ARRAY"));
	strings.insert(mypair(ASSERT,"ASSERT"));
	strings.insert(mypair(ATTRIBUTE,"ATTRIBUTE"));
	strings.insert(mypair(BEGIN,"BEGIN"));
	strings.insert(mypair(BLOCK,"BLOCK"));
	strings.insert(mypair(BODY,"BODY"));
	strings.insert(mypair(BUFFER,"BUFFER"));
	strings.insert(mypair(BUS,"BUS"));
	strings.insert(mypair(CASE,"CASE"));
	strings.insert(mypair(COMPONENT,"COMPONENT"));
	strings.insert(mypair(CONFIGURATION,"CONFIGURATION"));
	strings.insert(mypair(CONSTANT,"CONSTANT"));
	strings.insert(mypair(DISCONNECT,"DISCONNECT"));
	strings.insert(mypair(DOWNTO,"DOWNTO"));
	strings.insert(mypair(ELSE,"ELSE"));
	strings.insert(mypair(ELSIF,"ELSIF"));
	strings.insert(mypair(END,"END"));
	strings.insert(mypair(ENTITY,"ENTITY"));
	strings.insert(mypair(EXIT,"EXIT"));
	strings.insert(mypair(FILE,"FILE"));
	strings.insert(mypair(FOR,"FOR"));
	strings.insert(mypair(FUNCTION,"FUNCTION"));
	strings.insert(mypair(GENERATE,"GENERATE"));
	strings.insert(mypair(GENERIC,"GENERIC"));
	strings.insert(mypair(GROUP,"GROUP"));
	strings.insert(mypair(GUARDED,"GUARDED"));
	strings.insert(mypair(IF,"IF"));
	strings.insert(mypair(IMPURE,"IMPURE"));
	strings.insert(mypair(IN,"IN"));
	strings.insert(mypair(INERTIAL,"INERTIAL"));
	strings.insert(mypair(INOUT,"INOUT"));
	strings.insert(mypair(IS,"IS"));
	strings.insert(mypair(LABEL,"LABEL"));
	strings.insert(mypair(LIBRARY,"LIBRARY"));
	strings.insert(mypair(LINKAGE,"LINKAGE"));
	strings.insert(mypair(LITERAL,"LITERAL"));
	strings.insert(mypair(LOOP,"LOOP"));
	strings.insert(mypair(MAP,"MAP"));
	strings.insert(mypair(MOD,"MOD"));
	strings.insert(mypair(NAND,"NAND"));
	strings.insert(mypair(NEW,"NEW"));
	strings.insert(mypair(NEXT,"NEXT"));
	strings.insert(mypair(NOR,"NOR"));
	strings.insert(mypair(NOT,"NOT"));
	//strings.insert(mypair(NULL,"NULL"));
	strings.insert(mypair(OF,"OF"));
	strings.insert(mypair(ON,"ON"));
	strings.insert(mypair(OPEN,"OPEN"));
	strings.insert(mypair(OR,"OR"));
	strings.insert(mypair(OTHERS,"OTHERS"));
	strings.insert(mypair(OUT,"OUT"));
	strings.insert(mypair(PACKAGE,"PACKAGE"));
	strings.insert(mypair(PORT,"PORT"));
	strings.insert(mypair(POSTPONED,"POSTPONED"));
	strings.insert(mypair(PROCEDURE,"PROCEDURE"));
	strings.insert(mypair(PROCESS,"PROCESS"));
	strings.insert(mypair(PURE,"PURE"));
	strings.insert(mypair(RANGE,"RANGE"));
	strings.insert(mypair(RECORD,"RECORD"));
	strings.insert(mypair(REGISTER,"REGISTER"));
	strings.insert(mypair(REJECT,"REJECT"));
	strings.insert(mypair(RETURN,"RETURN"));
	strings.insert(mypair(ROL,"ROL"));
	strings.insert(mypair(ROR,"ROR"));
	strings.insert(mypair(SELECT,"SELECT"));
	strings.insert(mypair(SEVERITY,"SEVERITY"));
	strings.insert(mypair(SIGNAL,"SIGNAL"));
	strings.insert(mypair(SHARED,"SHARED"));
	strings.insert(mypair(SLA,"SLA"));
	strings.insert(mypair(SLI,"SLI"));
	strings.insert(mypair(SRA,"SRA"));
	strings.insert(mypair(SRL,"SRL"));
	strings.insert(mypair(SUBTYPE,"SUBTYPE"));
	strings.insert(mypair(THEN,"THEN"));
	strings.insert(mypair(TO,"TO"));
	strings.insert(mypair(TRANSPORT,"TRANSPORT"));
	strings.insert(mypair(TYPE,"TYPE"));
	strings.insert(mypair(UNAFFECTED,"UNAFFECTED"));
	strings.insert(mypair(UNITS,"UNITS"));
	strings.insert(mypair(UNTIL,"UNTIL"));
	strings.insert(mypair(USE,"USE"));
	strings.insert(mypair(VARIABLE,"VARIABLE"));
	strings.insert(mypair(WAIT,"WAIT"));
	strings.insert(mypair(WHEN,"WHEN"));
	strings.insert(mypair(WHILE,"WHILE"));
	strings.insert(mypair(WITH,"WITH"));
	strings.insert(mypair(XNOR,"XNOR"));
	strings.insert(mypair(XOR,"XOR"));
		

			//Types
	strings.insert(mypair(INTEGER,"integer"));
	strings.insert(mypair(STD_LOGIC,"std_logic"));
	strings.insert(mypair(_VECTOR,"_vector"));
	strings.insert(mypair(BIT,"bit"));
				
			//Operator
	strings.insert(mypair(ASSIGN_V," := "));
	strings.insert(mypair(ASSIGN_S," <= "));
	strings.insert(mypair(PLUS," + "));
	strings.insert(mypair(MINUS," - "));
	strings.insert(mypair(EQU," = "));
	strings.insert(mypair(NEQU," /= "));
	strings.insert(mypair(GEQU," >= "));
	strings.insert(mypair(LEQU," <= "));
	strings.insert(mypair(GREATER," > "));
	strings.insert(mypair(LOWER," < "));
	strings.insert(mypair(CONCAT," & "));
}

static std::string toBinary(int nr, int bits){
	std::string str;
	
	for(int i = bits-1; i>=0 ; i--){
		if(nr & (1<<i))str += "1";
		else str += "0";
	}
	return str;
}

void VHDLgen::WriteEntity(std::ofstream & file)
{
	using namespace std;
	int Wo = source.getWo();
	int Wi = source.getWi();
	
	file << "ENTITY " <<  userStrings[ENTITY_NAME]<< " IS" << endl;
	//file << "	GENERIC(" << userStrings[GENERIC_NAME] << " : integer := " << Wo << ");" << endl;
	file << "	PORT(" << endl;
	file << "		" << userStrings[INPUT_NAME] << " : IN std_logic_vector(" << (Wi - 1) << " DOWNTO 0);" << endl;
	file << "		" << userStrings[OUTPUT_NAME] << " : OUT std_logic_vector(" << (Wo - 1) << " DOWNTO 0);" << endl;
	file << "		clock : IN std_logic);" << endl;
	file << "END " << userStrings[ENTITY_NAME] <<";"<< endl;
}


void VHDLgen::WriteTIV(std::ofstream& file){
	using namespace std;
	
	int entryEachLine;
	int Width = source.getWidthTIV();
	int entrys = (1<<source.getAlpha());	
	int * TIVdata = new int[entrys];
	
	entryEachLine = 80/(Width+4);
	
	source.getTIV(TIVdata, entrys);
	
	file << "	CONSTANT tiv_rom : tiv_rom_t := (" << endl;
	
	for(int i = 0; i < entrys ; i += entryEachLine){
		file << "		";
		for(int j = i ; j < i+entryEachLine && j < entrys-1;j++){
			file << "\"" << toBinary(TIVdata[j], Width)  << "\", ";
		}
		if(i<entrys - entryEachLine)file << endl;
	}
	file << "\"" << toBinary(TIVdata[entrys-1],Width) << "\");" << endl; 
	
	delete [] TIVdata;
}


void VHDLgen::WriteTO(std::ofstream& file, int to_nr, bool symmetry)
{
	using namespace std;
	int entryEachLine, width;
	int beta, gamma, p;
	unsigned int mask;
	int sym = 0;
	int * TOdata;
	
	if(symmetry)sym = 1; 
	
	source.getTOparam(to_nr, gamma, beta, p);
	
	int entrys = (1<<(gamma + beta - sym));
	TOdata = new int[entrys];
	source.getTO(to_nr, TOdata, entrys, symmetry);
	
	width = (source.getWidthTO(to_nr) - sym);
	entryEachLine = 80 / (width + 4);
	
	mask = (1 << (source.getWidthTO(to_nr) - sym))-1;
	
	file << "	CONSTANT to" << to_nr << "_rom : to" << to_nr << "_rom_t := (" << endl;

	for(int i = 0; i < entrys ; i += entryEachLine){
		file << "		";
		for(int j = i ; j < i+entryEachLine && j < entrys-1;j++){
			file << "\"" << toBinary((TOdata[j]&mask), width)  << "\", ";
		}
		if(i<entrys - entryEachLine)file << endl;
	}
	file << "\"" << toBinary((TOdata[entrys-1]&mask), width) <<  "\");" << endl; 
	
	delete [] TOdata;
}

void VHDLgen::WriteTOProcess(std::ofstream& file, int to_nr, bool symmetry)
{
	using namespace std;
	int gamma, beta, p, wi, wo, guard;
	
	int sym = 1;
	//if(!symmetry)sym =0;
	
	source.getTOparam(to_nr, gamma, beta, p);
	wi = source.getWi();
	wo = source.getWo();
	guard = source.getGuardBits();
	
	file << "	--process reading TO" << to_nr << "_rom table" << endl;
	file << "	PROCESS(" << userStrings[INPUT_NAME] << ")" << endl;
	file << "		VARIABLE beta : std_logic_vector(" << (beta-2) << " DOWNTO 0);" << endl;
	file << "		VARIABLE data : std_logic_vector(" << (wo + guard) << " DOWNTO 0);" << endl;
	file << "	BEGIN" << endl;
	file << "		FOR i IN beta'range LOOP" << endl;
	file << "			beta(i) := " << userStrings[INPUT_NAME] << "(" << p << " + i) XOR NOT(" 
				<<userStrings[INPUT_NAME] << "(" << (beta+p-1) <<"));" << endl;
	file << "		END LOOP;" << endl << endl;
	
	file << "		data := \"";
	for(int i = 0;i<(wo + guard) - (source.getWidthTO(to_nr) -sym - 1);i++){file << "1";}
	file << "\" & to" << to_nr << "_rom(conv_integer(" << userStrings[INPUT_NAME] << "(" << wi-1 <<" DOWNTO " << (wi-gamma) << ") & beta));"<< endl;
	file << endl;
	
	file << "		FOR i IN data'range LOOP" << endl;
	file << "			data(i) := data(i) XOR NOT(" << userStrings[INPUT_NAME] << "(" << (beta+p-1) <<"));" << endl;
	file << "		END LOOP;" << endl<<endl;
	
	file << "		to" << to_nr << "_data <= data;" << endl;		
	file << "	END PROCESS;" << endl;
	
}


bool VHDLgen::Generate(std::string vhdl_path, bool symmetry)
{
	using namespace std;
	
	int sym = 0;	
	int Wo, alpha, beta, gamma, p, tos, guards;
	std::string sConcat;
	std::ofstream file(vhdl_path.c_str(), std::ios_base::out);
	
	if(symmetry)sym = 1;
	
	if(!file.is_open())return false;
		
	tos = source.getmTOs();
	Wo = source.getWo();
	alpha = source.getAlpha();
	guards = source.getGuardBits();
	
	file << "--Autogenerated Table lockup module" << endl << endl;
	
	file << "library IEEE;" << endl; 
	file << "use IEEE.STD_LOGIC_1164.ALL;" << endl;
	file << "use IEEE.STD_LOGIC_ARITH.ALL;" << endl;
	file << "use IEEE.STD_LOGIC_UNSIGNED.ALL;" << endl<<endl;
	
	WriteEntity(file);
	file<<endl<<endl;
	
	file << "ARCHITECTURE " << userStrings[ARCHITECTURE_NAME] << " OF " << userStrings[ENTITY_NAME]<< " IS" << endl;
	
	file << "	TYPE tiv_rom_t IS ARRAY(0 TO " << (1<<alpha)-1 << ") OF std_logic_vector(" << Wo + guards - 1<< " DOWNTO 0);" << endl;
	for(int i = tos-1; i >= 0 ; i--){
		source.getTOparam(i, gamma, beta, p);
		file << "	TYPE to" << i << "_rom_t IS ARRAY(0 TO " << (1<<gamma + beta - sym)-1 << ") OF std_logic_vector(" << source.getWidthTO(i)-sym - 1 << " DOWNTO 0);" << endl;
	}
	
	file<<endl;
	WriteTIV(file);
	file<<endl;
	
	for(int i=tos-1;i >= 0 ;i--){
		WriteTO(file, i, symmetry);
		file << endl;
	}
	
	file << "	SIGNAL tiv_data : std_logic_vector(" << (Wo + guards) << " DOWNTO 0);" << endl;
	for(int i = tos-1;i>= 0 ;i--){
		file << "	SIGNAL to" << i << "_data : std_logic_vector(" << (Wo + guards) << " DOWNTO 0);" << endl;
	}
	
	file << endl << "BEGIN" << endl;
	
	file << "	tiv_data <= '1' & tiv_rom(conv_integer(" << userStrings[INPUT_NAME] << "(" << (Wo - 1) << " DOWNTO " <<  (Wo - alpha) << ")));" << endl;
	file << endl;
	
	for(int i=tos-1;i >= 0 ;i--){
		WriteTOProcess(file, i, symmetry);
		file << endl;
	}	
	
	file << "	PROCESS(clock)" << endl;
	file << "		VARIABLE sum : std_logic_vector(" << (Wo + guards) << " DOWNTO 0);" << endl;
	file << "	BEGIN" << endl;
	file << "		IF clock'event AND clock = '1' THEN" << endl;
	file << "			sum := tiv_data";
	for(int i = tos-1;i>=0;i--){file << " + to" << i << "_data";} file << ";" << endl;
	file << "			" << userStrings[OUTPUT_NAME] << " <= sum(" << (Wo + guards) << " DOWNTO " << (guards+1) << ");" << endl;
	file << "		END IF;" << endl;
	file << "	END PROCESS;" << endl;
	file << "END " << userStrings[ARCHITECTURE_NAME] << ";" << endl;
	
	file.close();
	
	
	
	return true;	
}
