#pragma once
#include <string>
#include <map>
#include "Decomposition.h"

class VHDLgen
{
	public:
		VHDLgen(const Decomposition& source);
		~VHDLgen();
	
		bool Generate(std::string vhdl_path, bool symmetry = true);
		
	private:
		void WriteEntity(std::ofstream& file);
		void WriteTIV(std::ofstream& file);
		void WriteTO(std::ofstream& file, int to_nr, bool symmetry);
		void WriteTOProcess(std::ofstream& file, int to_nr, bool symmetry);
		
		enum {
			NO_KEY = 0,
			ABS,
			ACCESS,
			AFTER,
			ALIAS,
			ALL,
			AND,
			ARCHITECTURE,
			ARRAY,
			ASSERT,
			ATTRIBUTE,
			BEGIN,
			BLOCK,
			BODY,
			BUFFER,
			BUS,
			CASE,
			COMPONENT,
			CONFIGURATION,
			CONSTANT,
			DISCONNECT,
			DOWNTO,
			ELSE,
			ELSIF,
			END,
			ENTITY,
			EXIT,
			FILE,
			FOR,
			FUNCTION,
			GENERATE,
			GENERIC,
			GROUP,
			GUARDED,
			IF,
			IMPURE,
			IN,
			INERTIAL,
			INOUT,
			IS,
			LABEL,
			LIBRARY,
			LINKAGE,
			LITERAL,
			LOOP,
			MAP,
			MOD,
			NAND,
			NEW,
			NEXT,
			NOR,
			NOT,
			//NULL, compile error, NULL already exists
			OF,
			ON,
			OPEN,
			OR,
			OTHERS,
			OUT,
			PACKAGE,
			PORT,
			POSTPONED,
			PROCEDURE,
			PROCESS,
			PURE,
			RANGE,
			RECORD,
			REGISTER,
			REJECT,
			RETURN,
			ROL,
			ROR,
			SELECT,
			SEVERITY,
			SIGNAL,
			SHARED,
			SLA,
			SLI,
			SRA,
			SRL,
			SUBTYPE,
			THEN,
			TO,
			TRANSPORT,
			TYPE,
			UNAFFECTED,
			UNITS,
			UNTIL,
			USE,
			VARIABLE,
			WAIT,
			WHEN,
			WHILE,
			WITH,
			XNOR,
			XOR,
		

			//Types
			INTEGER,
			STD_LOGIC,
			_VECTOR,
			BIT,
			
			//Operator
			ASSIGN_V,
			ASSIGN_S,
			PLUS,
			MINUS,
			EQU,
			NEQU,
			GEQU,
			LEQU,
			GREATER,
			LOWER,
			CONCAT,
			END_KEYS
		}KeyWords;
		
		enum {
			ENTITY_NAME,
			ARCHITECTURE_NAME,
			INPUT_NAME,
			OUTPUT_NAME,
			GENERIC_NAME
		}USER_KEYS;
			
			
		static int class_count;
		static std::map<int, std::string> strings;
		std::map<int, std::string> userStrings;
		
		const Decomposition& source;
		
		
		static void generateStringMap();
			
};
