--Autogenerated Table lockup module

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY table_loockup IS
	PORT(
		data_in : IN std_logic_vector(7 DOWNTO 0);
		data_out : OUT std_logic_vector(7 DOWNTO 0);
		clock : IN std_logic);
END table_loockup;


ARCHITECTURE behaviour OF table_loockup IS
	TYPE tiv_rom_t IS ARRAY(0 TO 7) OF std_logic_vector(10 DOWNTO 0);
	TYPE to1_rom_t IS ARRAY(0 TO 31) OF std_logic_vector(7 DOWNTO 0);
	TYPE to0_rom_t IS ARRAY(0 TO 15) OF std_logic_vector(4 DOWNTO 0);

	CONSTANT tiv_rom : tiv_rom_t := (
		"11100100111", "10110001100", "10001000001", "01100101111", "01001001001", 
		"00110000110", "00011011101", "00001001011");

	CONSTANT to1_rom : to1_rom_t := (
		"11100011", "10101001", "01101111", "00110110", "11101000", "10111010", 
		"10001100", "01011110", "11101101", "11000111", "10100001", "01111100", 
		"11110000", "11010000", "10110001", "10010010", "11110010", "11011000", 
		"10111101", "10100011", "11110100", "11011101", "11000111", "10110000", 
		"11110110", "11100010", "11001110", "10111011", "11110111", "11100110", 
		"11010100", "11000011");

	CONSTANT to0_rom : to0_rom_t := (
		"11000", "01010", "11010", "01110", "11011", "10001", "11100", "10100", 
		"11100", "10110", "11101", "10111", "11101", "11000", "11101", "11001");

	SIGNAL tiv_data : std_logic_vector(11 DOWNTO 0);
	SIGNAL to1_data : std_logic_vector(11 DOWNTO 0);
	SIGNAL to0_data : std_logic_vector(11 DOWNTO 0);

BEGIN
	tiv_data <= '1' & tiv_rom(conv_integer(data_in(7 DOWNTO 5)));

	--process reading TO1_rom table
	PROCESS(data_in)
		VARIABLE beta : std_logic_vector(1 DOWNTO 0);
		VARIABLE data : std_logic_vector(11 DOWNTO 0);
	BEGIN
		FOR i IN beta'range LOOP
			beta(i) := data_in(2 + i) XOR NOT(data_in(4));
		END LOOP;

		data := "1111" & to1_rom(conv_integer(data_in(7 DOWNTO 5) & beta));

		FOR i IN data'range LOOP
			data(i) := data(i) XOR NOT(data_in(4));
		END LOOP;

		to1_data <= data;
	END PROCESS;

	--process reading TO0_rom table
	PROCESS(data_in)
		VARIABLE beta : std_logic_vector(0 DOWNTO 0);
		VARIABLE data : std_logic_vector(11 DOWNTO 0);
	BEGIN
		FOR i IN beta'range LOOP
			beta(i) := data_in(0 + i) XOR NOT(data_in(1));
		END LOOP;

		data := "1111111" & to0_rom(conv_integer(data_in(7 DOWNTO 5) & beta));

		FOR i IN data'range LOOP
			data(i) := data(i) XOR NOT(data_in(1));
		END LOOP;

		to0_data <= data;
	END PROCESS;

	PROCESS(clock)
		VARIABLE sum : std_logic_vector(11 DOWNTO 0);
	BEGIN
		IF clock'event AND clock = '1' THEN
			sum := tiv_data + to1_data + to0_data;
			data_out <= sum(11 DOWNTO 4);
		END IF;
	END PROCESS;
END behaviour;
