------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



package NR_implement_modules is
	--Operand normalizing module
	component norm_shifter is
		generic(
			pipe_stages :	integer := 4);
		port ( 
			dividend_in :	in std_logic_vector(63 downto 0);
			divisor_in :	in std_logic_vector(63 downto 0);
			enable :		in std_logic;
			reset : 		in std_logic;
			clock :			in std_logic;

			divisor_norm : 	out	std_logic_vector(63 downto 0);
			dividend_out :	out std_logic_vector(63 downto 0);
			divisor_out :	out std_logic_vector(63 downto 0);
			count : 		out	integer range 0 to 63);
	end component;
	
	--Lookup table used to get an initial approximation
	component lookup_table is
		generic(
			prec : integer := 4);
		port(
			dividend_in :		in std_logic_vector(63 downto 0);
			divisor_in :		in std_logic_vector(63 downto 0);
			divisor_norm_in :	in std_logic_vector(63 downto 0);
			count_in :	 		in integer range 0 to 63;
			enable :			in std_logic;
			reset :				in std_logic;
			clock :				in std_logic;
			
			dividend_out :		out std_logic_vector(63 downto 0);
			divisor_out :		out std_logic_vector(63 downto 0);
			divisor_norm_out :	out std_logic_vector(63 downto 0);
			count_out :	 		out integer range 0 to 63;
			resiprocal : 		out std_logic_vector(prec - 1 downto 0));
	
	end component;
	
	--One newton raphson stage implemented with redused pipelining for small slow stages
	component NR_stage_nopipe is
		generic(
			input_presicion : integer := 32);
		port(
			divisor :			in std_logic_vector(63 downto 0);	--!<the divior, fult width, internaly truncated
			resiprocal_in :		in std_logic_vector(input_presicion - 1 downto 0);	--!<initial resiprocal approximation
			enable :			in std_logic;		--!< clock enable
			reset :				in std_logic;
			clock :				in std_logic;		--!<the clock
			
			resiprocal_out :	out std_logic_vector((input_presicion * 2) - 1 downto 0));	--!<output resiprocal approximation
	end component;
	
	--One newton raphson stage implemented wiht posibilty for deep pipelining
	component NR_stage is
		generic(
			input_presicion : integer := 4;
			pipe_stages : integer := 2);
		port(
			divisor :			in std_logic_vector(63 downto 0);
			resiprocal_in :		in std_logic_vector(input_presicion - 1 downto 0);
			enable :			in std_logic;
			reset :				in std_logic;
			clock :				in std_logic;
			
			resiprocal_out :	out std_logic_vector((input_presicion * 2) - 1 downto 0));
	end component;
	
	--Wrapper for all implemented newton raphson stages
	component NR_stages is
		port(
			dividend_in :		in std_logic_vector(63 downto 0);
			divisor_in :		in std_logic_vector(63 downto 0);
			divisor_norm_in :	in std_logic_vector(63 downto 0);
			resiprocal_in :		in std_logic_vector(3 downto 0);
			shift_count_in :	in integer range 0 to 63;
			enable :			in std_logic;
			reset :				in std_logic;
			clock :				in std_logic;
			
			dividend_out :		out std_logic_vector(63 downto 0);
			divisor_out :		out std_logic_vector(63 downto 0);
			shift_count_out :	out integer range 0 to 63;
			resiprocal_out :	out std_logic_vector(63 downto 0);
			div_by_zero :		out std_logic);
	end component;
	
	--Quotient calculation module
	component quotient_calc is
		generic(pipe_stages1 : integer := 4;
				pipe_stages2 : integer := 4);
		port(
			resciprocal :	in std_logic_vector(63 downto 0);
			dividend_in :	in std_logic_vector(63 downto 0);
			divisor_in :	in std_logic_vector(63 downto 0);
			div_by_zero_in :in std_logic;
			shift_count :	in integer range 63 downto 0;
			enable :		in std_logic;
			reset :			in std_logic;
			clock :			in std_logic;
			
			dividend_out :	out std_logic_vector(63 downto 0);
			divisor_out :	out std_logic_vector(63 downto 0);
			div_by_zero_out:out std_logic;
			quotient : 		out std_logic_vector(63 downto 0));
	end component;
	
	--Remainder calculation module
	component rest_calc is
		generic( pipe_stages : integer := 4);
		port(
			quotient_in : 	in std_logic_vector(63 downto 0);
			dividend_in : 	in std_logic_vector(63 downto 0);
			divisor_in: 	in std_logic_vector(63 downto 0);
			div_by_zero_in:	in std_logic;
			enable : 		in std_logic;
			reset : 		in std_logic;
			clock : 		in std_logic;
		
			quotient_out :	out std_logic_vector(63 downto 0);
			divisor_out :	out std_logic_vector(63 downto 0);
			div_by_zero_out:out std_logic;
			rest : 			out std_logic_vector(63 downto 0));
	end component;
	
	--Final adjustment stage
	component round is
	port(
		approx_quot : in	std_logic_vector(63 downto 0);
		approx_rest : in	std_logic_vector(63 downto 0);
		divisor : in		std_logic_vector(63 downto 0);
		zero : in			std_logic;
		reset : in std_logic;
		
		quotient : out		std_logic_vector(63 downto 0);
		rest : out			std_logic_vector(63 downto 0));
	end component;
	
	--auxiliary functions 
	function min(a, b : integer)return integer;
	
end package;

package body NR_implement_modules is

	function Min(a, b : integer)return integer is
	begin
		if a > b then
			return b;
		else
			return a;
		end if;
	end Min;

end package body;
