------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Implementes high throughput pipelined division circuit based on Newton-Rahsons method
------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.numeric_std.all;

library work;
use work.NR_implement_modules.all;

entity divide is
	generic( width : integer := 64);
	port(
		dividend : in std_logic_vector(width - 1 downto 0);
		divisor : in std_logic_vector(width - 1 downto 0);	
		enable : in std_logic;
		reset : in std_logic;
		clock : in std_logic;
		
		quotient : out std_logic_vector(width - 1 downto 0);
		rest : out std_logic_vector(width - 1 downto 0));
end divide;

architecture behavior of divide is
	signal divisor_norm1 : std_logic_vector(63 downto 0);
	signal divisor1 : std_logic_vector(63 downto 0);
	signal dividend1 : std_logic_vector(63 downto 0);
	signal count1 : integer range 0 to 63;
	
	signal divisor_norm2 : std_logic_vector(63 downto 0);
	signal divisor2 : std_logic_vector(63 downto 0);
	signal dividend2 : std_logic_vector(63 downto 0);
	signal count2 : integer range 0 to 63;
	signal resiprocal2 : std_logic_vector(3 downto 0);
	
	signal divisor3 : std_logic_vector(63 downto 0);
	signal dividend3 : std_logic_vector(63 downto 0);
	signal count3 : integer range 0 to 63;
	signal resiprocal3 : std_logic_vector(63 downto 0);
	signal div_by_zero3 : std_logic;
	
	signal divisor4 : std_logic_vector(63 downto 0);
	signal dividend4 : std_logic_vector(63 downto 0);
	signal div_by_zero4 : std_logic;
	signal quotient4 : std_logic_vector(63 downto 0);

	signal divisor5 : std_logic_vector(63 downto 0);
	signal rest5 : std_logic_vector(63 downto 0);
	signal div_by_zero5 : std_logic;
	signal quotient5 : std_logic_vector(63 downto 0);
		
begin

	--Normalize input divisor
	input_shift:component norm_shifter
		generic map(1)
		port map(dividend, divisor, enable, reset, clock,
				divisor_norm1, dividend1, divisor1, count1);
	
	--Get an initial approximation
	table_lookup:component lookup_table
		generic map(4)
		port map(dividend1, divisor1, divisor_norm1, count1, enable, reset, clock, 
				dividend2, divisor2, divisor_norm2, count2, resiprocal2);
				
		
	--NewtonRaphson stages
	NewtonRaphson:component NR_stages
		port map(dividend2, divisor2, divisor_norm2, resiprocal2, count2, enable, reset, clock,
				dividend3, divisor3, count3, resiprocal3, div_by_zero3);

	--Calculate quotient
	clac_quot:component quotient_calc
		generic map (1,1)
		port map(resiprocal3, dividend3, divisor3, div_by_zero3, count3, enable, reset, clock,
				dividend4, divisor4, div_by_zero4, quotient4);
	
	
	--Calculate the remainder
	calc_rest:component rest_calc
		generic map (1)
		port map(quotient4, dividend4, divisor4, div_by_zero4,  enable, reset, clock,
				quotient5, divisor5, div_by_zero5, rest5);


	--Final rounding to achieve 1ulp precision
	rounder:component round
		port map(quotient5, rest5, divisor5, div_by_zero5, reset,
				quotient, rest);
				
end behavior;
		
