------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Implements one Newton-Rapson stage
------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.numeric_std.ALL;
library work;
use work.NR_implement_modules.min;


entity NR_stage is
	generic(
		input_presicion : integer := 32;
		pipe_stages : integer := 20);--Pipeline stages to push in to the stage
	port(
		divisor :			in std_logic_vector(63 downto 0);	--!<the divior, fult width, internaly truncated
		resiprocal_in :		in std_logic_vector(input_presicion - 1 downto 0);	--!<initial resiprocal approximation
		enable :			in std_logic;		--!< clock enable
		reset :				in std_logic;
		clock :				in std_logic;		--!<the clock
		
		resiprocal_out :	out std_logic_vector((input_presicion * 2) - 1 downto 0));	--!<output resiprocal approximation
end NR_stage;


architecture behavior of NR_stage is
	constant output_presicion : integer := input_presicion * 2;
	constant trunc_div_width : integer := work.NR_implement_modules.min(divisor'length, output_presicion + 2 + 1);
	
	
	subtype mult_one_range is integer range trunc_div_width + resiprocal_in'length - 1 downto 0;
	type mult_one_pipe_t is array (0 to pipe_stages/2 - 1) of std_logic_vector(mult_one_range);
	
	type resi_pipe_t is array (0 to pipe_stages/2 - 1) of std_logic_vector(resiprocal_in'range);
			
	signal trunc_divisor : std_logic_vector(trunc_div_width - 1 downto 0);
	signal mult_one_pipe : mult_one_pipe_t;	
	signal trunc_mult_one : std_logic_vector(output_presicion + 2 downto 0);
	signal komplement : std_logic_vector(output_presicion + 2 downto 0);
	signal resi_pipe : resi_pipe_t;

	subtype mult_two_range is integer range trunc_mult_one'length + resiprocal_in'length - 1 downto 0;
	type mult_two_pipe_t is array (0 to pipe_stages/2 - 1) of std_logic_vector(mult_two_range);
	
	signal mult_two_pipe : mult_two_pipe_t;
	 
begin
	--Truncate the input divisor to the needed bit width
	trunc_divisor <= divisor(divisor'high downto divisor'length - trunc_divisor'length);
	
	--implements the first multiplication
	mult_one:process(clock)
	begin	
		if clock'event and clock = '1' then
			if reset = '1' then
				mult_one_pipe <= (others => (others => '0'));		
			elsif enable = '1' then
			
				mult_one_pipe(0) <= trunc_divisor * resiprocal_in;
			
				for i in mult_one_pipe'high downto 1 loop
					mult_one_pipe(i) <= mult_one_pipe(i-1);
				end loop;
			end if;
		end if;
			
	end process;
	
	--Truncate the output from the first multiplier
	trunc_mult_one <= mult_one_pipe(mult_one_pipe'high)(mult_one_range'high downto mult_one_range'high - trunc_mult_one'high);
	
	--Pipe the input resiprocal forward to the second multiplier
	resi_pipe_flow:process(clock)
	begin

		if clock'event and clock = '1' then
			if reset = '1' then
				resi_pipe <= (others => (others => '0'));		
			elsif enable = '1' then
				resi_pipe(0) <= resiprocal_in;
			
				for i in resi_pipe'high downto 1 loop
					resi_pipe(i) <= resi_pipe(i - 1);
				end loop;
			end if;
		end if;
	end process;
	
	
	mult_two:process(clock)
		variable temp : std_logic_vector(trunc_mult_one'range);
	begin		
		if clock'event and clock = '1' then
			if reset = '1' then
				mult_two_pipe <= (others => (others => '0'));
			elsif enable = '1' then
				--Implements exstra pipeline stage after the subtraction
				--komplement <= not(trunc_mult_one) + '1';
				--mult_two_pipe(0) <= komplement * resi_pipe(resi_pipe'high);
				
				--Implements no pipeline stage after the subtraction
				temp := not(trunc_mult_one) + '1';	--two's complement
				mult_two_pipe(0) <= temp * resi_pipe(resi_pipe'high);
				
				for i in mult_two_pipe'high downto 1 loop
					mult_two_pipe(i) <= mult_two_pipe(i-1);
				end loop;
			end if;
		end if;
	end process;
	
	--Truncate the second multiplier
	resiprocal_out <= mult_two_pipe(mult_two_pipe'high)(mult_two_range'high - 1 downto mult_two_range'high - resiprocal_out'high - 1);

end behavior;


