------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Implements one Newton-Rapson stage with one pipeline stage for the entier NR stage
------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity NR_stage_nopipe is
	generic(
		input_presicion : integer := 32);
	port(
		divisor :			in std_logic_vector(63 downto 0);	--!<the divior, fult width, internaly truncated
		resiprocal_in :		in std_logic_vector(input_presicion - 1 downto 0);	--!<initial resiprocal approximation
		enable :			in std_logic;		--!< clock enable
		reset :				in std_logic;
		clock :				in std_logic;		--!<the clock
		
		resiprocal_out :	out std_logic_vector((input_presicion * 2) - 1 downto 0));	--!<output resiprocal approximation
end NR_stage_nopipe;

architecture Behavioral of NR_stage_nopipe is
	constant output_presicion : integer := input_presicion * 2;
	constant trunc_div_width : integer := work.NR_implement_modules.min(divisor'length, output_presicion + 2 + 1);
	
	
	subtype mult_one_range is integer range trunc_div_width + resiprocal_in'length - 1 downto 0;
	signal trunc_divisor : std_logic_vector(trunc_div_width - 1 downto 0);
	signal trunc_mult_one : std_logic_vector(output_presicion + 2 downto 0);

	subtype mult_two_range is integer range trunc_mult_one'length + resiprocal_in'length - 1 downto 0;
	 
begin
	
	trunc_divisor <= divisor(divisor'high downto divisor'length - trunc_divisor'length);
	
	mult_one:process(clock)
		variable mult_one : std_logic_vector(mult_one_range);
		variable trunc_mult_one : std_logic_vector(output_presicion + 2 downto 0);
		variable temp : std_logic_vector(trunc_mult_one'range);
		variable mult_two : std_logic_vector(mult_two_range);
	begin	
		if clock'event and clock = '1' then
			if reset = '1' then
				resiprocal_out <= (others => '0');			
			elsif enable = '1' then
			
				mult_one := trunc_divisor * resiprocal_in;
				trunc_mult_one := mult_one(mult_one'high downto mult_one'high - trunc_mult_one'high);
			
				temp := not(trunc_mult_one) + '1';
			
				mult_two := temp * resiprocal_in;	
				resiprocal_out <= mult_two(mult_two'high - 1 downto mult_two_range'high - resiprocal_out'high - 1);

			end if;
		end if;		
	end process;

end Behavioral;

