------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Test bench used to test entire circuit
------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
--use ieee. std_logic_arith.all;
use IEEE.numeric_std.ALL;
library work;
use work.txt_util.all;

entity NewtonSim_tb is end NewtonSim_tb;

architecture behavior of NewtonSim_tb is

	component divide is
	generic( width : integer := 64);
	port(
		dividend : in std_logic_vector(width - 1 downto 0);
		divisor : in std_logic_vector(width - 1 downto 0);
		enable : in std_logic;
		reset : in std_logic;
		clock : in std_logic;
		
		quotient : out std_logic_vector(width - 1 downto 0);
		rest : out std_logic_vector(width - 1 downto 0));
   end component;

	
	constant periode : time := 10 ns;
	
	signal tb_dividend : std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor : std_logic_vector(63 downto 0) := (others => '0');
	signal tb_enable : std_logic := '0';
	signal tb_reset : std_logic := '0';
	signal tb_clock : std_logic := '0';
	signal tb_quotient: std_logic_vector(63 downto 0);
	signal tb_rest : std_logic_vector(63 downto 0);
	
	type pipe is array (0 to 36) of std_logic_vector(63 downto 0);
	signal dividend_pipe : pipe := (others => (others => '0'));
	signal divisor_pipe : pipe := (others => (others => '0'));
	signal valid_pipe : std_logic_vector(0 to 36) := (others =>'0');
	signal valid : std_logic := '0';
	signal ans_rest, ans_quotient : std_logic_vector(63 downto 0) := (others => '0');
	
begin
	
	dut:divide port map(
		tb_dividend,
		tb_divisor,
		tb_enable,
		tb_reset,
		tb_clock,
		tb_quotient,
		tb_rest);
		
	process
	begin
	   wait for periode/2;
	   tb_clock <= '0';
	   wait for periode/2;
	   tb_clock <= '1';
	end process;
		
	
	process
	    variable divd, divs : std_logic_vector(tb_dividend'range);
	    variable pattern : std_logic_vector(tb_dividend'range);
	begin
		tb_reset <= '1';
	   wait until tb_clock = '1';
	   wait for periode/2;
	   
	   tb_reset <= '0';
	   
	   wait for periode;
	   
	   tb_enable <= '1';
	   divd := (others => '0');
	   divd(0) := '1';
	   
	   divs := (others => '0');
	   divs(0) := '1';
	   divs(1) := '1';
	   
	   tb_dividend <= divd;
	   tb_divisor  <= divs;
	   wait for periode;
      
	   divd := (others => '0');
	   divs := (others => '0');
	   divs(0) := '1';
	   tb_dividend <= divd;
	   tb_divisor  <= divs;
      wait for periode;
	   
	   --Walking once dividend and divisor
	   for i in 0 to 63 loop
	      divd := (others => '0');
	      divd(i) := '1';
	      for j in 0 to 63 loop
	         divs := (others => '0');
	         divs(j) := '1';
	         
	         tb_dividend <= divd;
	         tb_divisor  <= divs;
	         
	         wait for periode;

	     end loop;
	     --report "Simulation progress!";
	   end loop;
	   
	   --walking pattern (high bit '1')
	   for i in 0 to 63 loop
	      divd := (others=>'0');
	      divd(divd'high) := '1';
	      divd(i) := '1';
	      for p_len in 1 to 62 loop
	         
	         pattern := (others => '0');
	         for j in 0 to p_len - 1 loop pattern(j) := '1'; end loop;
	         
	         for j in 0 to 64 - p_len loop
	            divs := (divs'high => '1', others => '0');
	            divs := divs or pattern;
	            pattern := pattern(pattern'high - 1 downto 0) & '0';
	            
	 	         tb_dividend <= divd;
	            tb_divisor  <= divs;
               
               wait for periode;
               
	         end loop;
	      end loop;
	   end loop;
	   
	   --walking pattern (high bit '1')
	   for i in 0 to 63 loop
	      divd := (others=>'0');
	      divd(divd'high) := '1';
	      divd(i) := '1';
	      for p_len in 1 to 62 loop
	         
	         pattern := (others => '1');
	         for j in 0 to p_len - 1 loop pattern(j) := '0'; end loop;
	         
	         for j in 0 to 64 - p_len loop
	            divs := (others => '1');
	            divs := divs and pattern;
	            pattern := pattern(pattern'high - 1 downto 0) & '1';
	            
	 	         tb_dividend <= divd;
	            tb_divisor  <= divs;
               
               wait for periode;
	         
	         end loop;
	      end loop;
	   end loop;
	   
	   tb_dividend <= (others => '0');
	   tb_divisor  <= (others => '0');
	   
	   for i in 0 to 36 loop
	      wait for periode;
	   end loop;
	   
	   tb_enable <= '0';
	   
	   assert (1 = 0)
	   report "Simulation ok!"
	   severity failure;
	   
	end process;
	
	
	temp_store:process(tb_clock)
	begin
		if tb_clock'event and tb_clock = '1' then

			dividend_pipe(0) <= tb_dividend;
			divisor_pipe(0) <= tb_divisor;
			valid_pipe(0) <= tb_enable;

			for i in 1 to dividend_pipe'high loop
				dividend_pipe(i) <= dividend_pipe(i-1);
				divisor_pipe(i) <= divisor_pipe(i-1);
				valid_pipe(i) <= valid_pipe(i-1);
			end loop;

			if valid_pipe(valid_pipe'high-1) = '1' then
				ans_rest <= std_logic_vector(unsigned(dividend_pipe(dividend_pipe'high-1)) rem unsigned(divisor_pipe(divisor_pipe'high-1)));
				ans_quotient <= std_logic_vector(unsigned(dividend_pipe(dividend_pipe'high-1)) / unsigned(divisor_pipe(divisor_pipe'high-1)));
				valid <= '1';
			else
				ans_rest <= (others => '0');
				ans_quotient <= (others => '0');
				valid <= '0'; 
			end if;     
		end if;
		
	end process;
	
	verify:process(tb_clock)
	   variable s, d : string(1 to 64);
	begin
	   if tb_clock'event and tb_clock = '0' then
		   if valid = '1' then	
			   if ans_rest /= tb_rest or ans_quotient /= tb_quotient then
				   report "Error";
			   end if;
		   end if;
	   end if;
	end process;
	    
		
end behavior;
