------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Implements 4 bit in 4 bit out reciprocal approximation
------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.numeric_std.all;

entity lookup_table is
	generic(
		prec : integer := 4);
	port(
			dividend_in :		in std_logic_vector(63 downto 0);
			divisor_in :		in std_logic_vector(63 downto 0);
			divisor_norm_in :	in std_logic_vector(63 downto 0);
			count_in :	 		in integer range 0 to 63;
			enable :			in std_logic;
			reset :				in std_logic;
			clock :				in std_logic;
			
			dividend_out :		out std_logic_vector(63 downto 0);
			divisor_out :		out std_logic_vector(63 downto 0);
			divisor_norm_out :	out std_logic_vector(63 downto 0);
			count_out :	 		out integer range 0 to 63;
			resiprocal : 		out std_logic_vector(prec - 1 downto 0));
		
		
end lookup_table;

architecture behavior of lookup_table is
	constant precision : integer := 4;
begin
	
	lookup:process(divisor_norm_in)
		variable trunc : std_logic_vector(precision-1 downto 0);
	begin
	
		trunc := divisor_norm_in(divisor_norm_in'high - 1 downto divisor_norm_in'length - precision - 1);
	
		case trunc is
			when "0000" => resiprocal <= "1111";--Rounded down on ULP to prevent overflow
			when "0001" => resiprocal <= "1111";
			when "0010" => resiprocal <= "1110";
			when "0011" => resiprocal <= "1101";
			when "0100" => resiprocal <= "1100";
			when "0101" => resiprocal <= "1100";
			when "0110" => resiprocal <= "1011";
			when "0111" => resiprocal <= "1011";
			when "1000" => resiprocal <= "1010";
			when "1001" => resiprocal <= "1010";
			when "1010" => resiprocal <= "1001"; 
			when "1011" => resiprocal <= "1001";
			when "1100" => resiprocal <= "1001";
			when "1101" => resiprocal <= "1000";
			when "1110" => resiprocal <= "1000";
			when others => resiprocal <= "1000"; 
		end case;
	end process;

	--Pipelining of side data
	dividend_out <= dividend_in;
	divisor_out <= divisor_in;
	divisor_norm_out <= divisor_norm_in;
	count_out <= count_in;	
	
end behavior;
		
