------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Implements the remainder calculation circuit
------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.numeric_std.all;

entity rest_calc is
	generic( pipe_stages : integer := 4);
	port(
		quotient_in : 	in std_logic_vector(63 downto 0);
		dividend_in : 	in std_logic_vector(63 downto 0);
		divisor_in: 	in std_logic_vector(63 downto 0);
		div_by_zero_in:	in std_logic;
		enable : 		in std_logic;
		reset :			in std_logic;
		clock : 		in std_logic;
	
		quotient_out :	out std_logic_vector(63 downto 0);
		divisor_out :	out std_logic_vector(63 downto 0);
		div_by_zero_out:out std_logic;
		rest : 			out std_logic_vector(63 downto 0));
end rest_calc;

architecture behavior of rest_calc is
	constant n : integer := 64;
	
	type mult_pipe_t is array (0 to pipe_stages - 1) of std_logic_vector((2*n)-1 downto 0);
	type pipe is array (0 to pipe_stages - 1) of std_logic_vector(63 downto 0);
	
	signal mult_pipe : mult_pipe_t;
	signal dividend_pipe : pipe;
	signal divisor_pipe : pipe;
	signal quotient_pipe : pipe;
	signal zero_pipe : std_logic_vector(0 to pipe_stages - 1);
	
begin
	
	--Implements pipelining of bypass data and a pipelined multiplier
	mult:process(clock)
	begin	
		if clock'event and clock = '1' then
			if reset = '1' then
				mult_pipe <= (others => (others => '0'));
					
				dividend_pipe <= (others => (others => '0'));
				
				divisor_pipe <= (others => (others => '0'));

				quotient_pipe <= (others => (others => '0'));
					
				zero_pipe <= (others => '0');
		
			elsif enable = '1' then
				mult_pipe(0) <= quotient_in * divisor_in;
				for i in 1 to mult_pipe'high loop
					mult_pipe(i) <= mult_pipe(i-1);
				end loop;
					
				dividend_pipe(0) <= dividend_in;
				for i in 1 to dividend_pipe'high loop
				   dividend_pipe(i) <= dividend_pipe(i-1);
				end loop;
			
				divisor_pipe(0) <= divisor_in;
				for i in 1 to divisor_pipe'high loop
				   divisor_pipe(i) <= divisor_pipe(i-1);
				end loop;
			
				quotient_pipe(0) <= quotient_in;
				for i in 1 to quotient_pipe'high loop
				   quotient_pipe(i) <= quotient_pipe(i-1);
				end loop;
			
				zero_pipe(0) <= div_by_zero_in;
				for i in 1 to zero_pipe'high loop
				   zero_pipe(i) <= zero_pipe(i-1);
				end loop;
			end if;					
		end if;
	end process;
	
	--Implements a final subtaction
	rest <= dividend_pipe(dividend_pipe'high) - mult_pipe(mult_pipe'high)(n-1 downto 0);
	
	divisor_out <= divisor_pipe(divisor_pipe'high);
	div_by_zero_out <= zero_pipe(zero_pipe'high);
	quotient_out <= quotient_pipe(quotient_pipe'high);
end behavior;

