------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Test bench used to test the Newton-Raphson stages
------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.numeric_std.all;

entity tb_NR_stages is
end tb_NR_stages;

architecture behavior of tb_NR_stages is
	
	constant periode : time := 10 ns;
	

	component NR_stages is
		port(
			dividend_in :		in std_logic_vector(63 downto 0);
			divisor_in :		in std_logic_vector(63 downto 0);
			divisor_norm_in :	in std_logic_vector(63 downto 0);
			resiprocal_in :		in std_logic_vector(3 downto 0);
			shift_count_in :	in integer range 0 to 63;
			enable :			in std_logic;
			clock :				in std_logic;
			
			dividend_out :		out std_logic_vector(63 downto 0);
			divisor_out :		out std_logic_vector(63 downto 0);
			shift_count_out :	out integer range 0 to 63;
			resiprocal_out :	out std_logic_vector(63 downto 0);
			div_by_zero :		out std_logic);
	end component;

	signal tb_dividend_in :		std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_in :		std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_norm_in :	std_logic_vector(63 downto 0) := (others => '0');
	signal tb_resiprocal_in :	std_logic_vector(3 downto 0) := (others => '0');
	signal tb_shift_count_in :	integer range 0 to 63 := 0;
	signal tb_enable :			std_logic := '0';
	signal tb_clock :			std_logic := '0';
			
	signal tb_dividend_out :	std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_out :		std_logic_vector(63 downto 0) := (others => '0');
	signal tb_shift_count_out :	integer range 0 to 63 := 0;
	signal tb_resiprocal_out :	std_logic_vector(63 downto 0) := (others => '0');
	signal tb_div_by_zero :		std_logic := '0';
	
begin


	dut:NR_stages
		port map(
			tb_dividend_in,
			tb_divisor_in,
			tb_divisor_norm_in,
			tb_resiprocal_in,
			tb_shift_count_in,
			tb_enable,
			tb_clock,
			
			tb_dividend_out,
			tb_divisor_out,
			tb_shift_count_out,
			tb_resiprocal_out,
			tb_div_by_zero);
			
	clocking:process
	begin
		wait for periode/2;
		tb_clock <= not(tb_clock);
	end process;
	
	data:process
		variable divd, divs : std_logic_vector(63 downto 0);
		variable temp : std_logic_vector(63 downto 0);
	begin
		wait until tb_clock'event and tb_clock = '1';
		
		tb_enable <= '1';
		
		wait until tb_clock'event and tb_clock = '1';
		
		divs := (others => '0');
		divs(divs'high) := '1';
		divs(divs'high-1) := '1';
		
		tb_divisor_norm_in <= divs;
		tb_resiprocal_in <= "1010";
		
		temp := (others => '0');
		
		temp(0) := '1';
		tb_divisor_in <= temp;
		
		temp(1) := '1';
		tb_dividend_in <= temp;
					
		tb_shift_count_in <= 4;
		
		wait until tb_clock'event and tb_clock = '1';
		
		tb_divisor_norm_in <= (others => '0');
		tb_resiprocal_in <= "0000";
		
		temp := (others => '0');
		tb_divisor_in <= temp;
		tb_dividend_in <= temp;
		tb_shift_count_in <= 0;
		wait;
	end process;
			
end behavior;
	
