------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Test bench used to test the lookup table
------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use IEEE.numeric_std.all;

entity tb_lookup_table is
end tb_lookup_table;

architecture behavior of tb_lookup_table is
	
	constant periode : time := 10 ns;
	
	component lookup_table is
		generic(prec : integer := 4);
		port(
			dividend_in :		in std_logic_vector(63 downto 0);
			divisor_in :		in std_logic_vector(63 downto 0);
			divisor_norm_in :	in std_logic_vector(63 downto 0);
			count_in :	 		in integer range 0 to 63;
			enable :			in std_logic;
			clock :				in std_logic;
			
			dividend_out :		out std_logic_vector(63 downto 0);
			divisor_out :		out std_logic_vector(63 downto 0);
			divisor_norm_out :	out std_logic_vector(63 downto 0);
			count_out :	 		out integer range 0 to 63;
			resiprocal : 		out std_logic_vector(prec - 1 downto 0));
	end component;

	signal tb_dividend_in :		std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_in :		std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_norm_in :	std_logic_vector(63 downto 0) := (others => '0');
	signal tb_count_in :	 	integer range 0 to 63 := 0;
	signal tb_enable :			std_logic := '0';
	signal tb_clock :			std_logic := '0';

	signal tb_dividend_out :	std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_out :		std_logic_vector(63 downto 0) := (others => '0');
	signal tb_divisor_norm_out :std_logic_vector(63 downto 0) := (others => '0');
	signal tb_count_out :	 	integer range 0 to 63  := 0;
	signal tb_resiprocal : 		std_logic_vector(3 downto 0) := (others => '0');
	
begin


	dut:lookup_table
		generic map(4)
		port map(
			tb_dividend_in,
			tb_divisor_in,
			tb_divisor_norm_in,
			tb_count_in,
			tb_enable,
			tb_clock,

			tb_dividend_out,
			tb_divisor_out,
			tb_divisor_norm_out,
			tb_count_out,
			tb_resiprocal);
			
	clock:process
	begin
		wait for periode/2;
		tb_clock <= not(tb_clock);
	end process;
	
	data:process
		variable divd, divs : std_logic_vector(63 downto 0);
	begin
		wait until tb_clock'event and tb_clock = '1';
		
		tb_enable <= '1';
		
		wait until tb_clock'event and tb_clock = '1';
		
		divd := (others => '0');
		divd(0) := '1';
		divs := (others => '0');
		divs(divs'high) := '1';
		
		tb_divisor_norm_in <= divs;
		tb_dividend_in <= divd;
		
		
		wait until tb_clock'event and tb_clock = '1';
		
		divd := (others => '0');
		divs(divs'high-1) := '1';
		
		tb_divisor_norm_in <= divs;
		tb_dividend_in <= divd;
				
		wait;
	end process;
			
end behavior;
