------------------------------------------------------
--(copy left) Arthur Øvergaard 
--Trondheim June 09
--
--Test bench used to test the quotient calculation circuit
------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.ALL;
 
ENTITY tb_quotient IS
END tb_quotient;
 
ARCHITECTURE behavior OF tb_quotient IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT quotient_calc
    PORT(
         resciprocal : IN  std_logic_vector(63 downto 0);
         dividend_in : IN  std_logic_vector(63 downto 0);
         divisor_in : IN  std_logic_vector(63 downto 0);
         div_by_zero_in : IN  std_logic;
         shift_count : in integer range 63 downto 0;
         enable : IN  std_logic;
         reset : IN std_logic;
         clock : IN  std_logic;
         dividend_out : OUT  std_logic_vector(63 downto 0);
         divisor_out : OUT  std_logic_vector(63 downto 0);
         div_by_zero_out : OUT  std_logic;
         quotient : OUT  std_logic_vector(63 downto 0)
        );
    END COMPONENT;
    

   --Inputs
   signal tb_resciprocal : std_logic_vector(63 downto 0) := (others => '0');
   signal tb_dividend_in : std_logic_vector(63 downto 0) := (others => '0');
   signal tb_divisor_in : std_logic_vector(63 downto 0) := (others => '0');
   signal tb_div_by_zero_in : std_logic := '0';
   signal tb_shift_count : integer range 63 downto 0 := 0;
   signal tb_enable : std_logic := '0';
   signal tb_reset : std_logic := '1';
   signal tb_clock : std_logic := '0';

 	--Outputs
   signal tb_dividend_out : std_logic_vector(63 downto 0);
   signal tb_divisor_out : std_logic_vector(63 downto 0);
   signal tb_div_by_zero_out : std_logic;
   signal tb_quotient : std_logic_vector(63 downto 0);

   -- Clock period definitions
   constant tb_clock_period : time := 10 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: quotient_calc PORT MAP (
          resciprocal => tb_resciprocal,
          dividend_in => tb_dividend_in,
          divisor_in => tb_divisor_in,
          div_by_zero_in => tb_div_by_zero_in,
          shift_count => tb_shift_count,
          enable => tb_enable,
          reset => tb_reset,
          clock => tb_clock,
          dividend_out => tb_dividend_out,
          divisor_out => tb_divisor_out,
          div_by_zero_out => tb_div_by_zero_out,
          quotient => tb_quotient
        );

   -- Clock process definitions
   clock_process :process
   begin
	
		wait for tb_clock_period / 2;
		tb_clock <= not(tb_clock);
   end process;
 

   -- Stimulus process
   stim_proc: process
		variable temp : std_logic_vector(63 downto 0);
   begin		
		
		wait until tb_clock'event and tb_clock = '1';
		tb_reset <= '0';
		wait for tb_clock_period / 2;
		
		tb_enable <= '1';
		temp := (others => '0');
		temp(60) := '1';
		
		tb_dividend_in <= temp;
		tb_resciprocal <= (others => '1');
		tb_shift_count <= 0;
		wait for tb_clock_period;
		
		for i in 45 to 63 loop
		   tb_shift_count <= i;
		   wait for tb_clock_period;
		end loop;
		

      wait;
   end process;

END;
